from ..widget_base import Input
from typing import List, Optional


class CurrencyInput(Input):
    type = "currency-input"
    value: Optional[float]
    empty_value = None

    def __init__(self, key: str, label: str, **kwargs):
        super().__init__(key)
        self.set_props(dict(label=label, **kwargs))

    def set_props(self, props):
        self.label = props.get("label", "Label")
        self.value = props.get("initial_value", self.empty_value)
        self.required = props.get("required", True)
        self.hint = props.get("hint", None)
        self.placeholder = props.get("placeholder", "")
        self.full_width = props.get("full_width", False)
        self.min = props.get("min")
        self.max = props.get("max")
        self.currency = props.get("currency", "USD")
        self.disabled = props.get("disabled", False)

    def _validate_number_min_max(self) -> List[str]:
        if type(self.value) != int and type(self.value) != float:
            return []
        if hasattr(self, "min") and self.min is not None and self.value < self.min:
            return ["i18n_error_min_amount"]
        if hasattr(self, "max") and self.max is not None and self.value > self.max:
            return ["i18n_error_max_amount"]
        return []

    @property
    def validators(self):
        return super().validators + [self._validate_number_min_max]

    def render(self, ctx: dict):
        return {
            "type": self.type,
            "key": self.key,
            "label": self.label,
            "value": self.serialize_value(),
            "placeholder": self.placeholder,
            "required": self.required,
            "hint": self.hint,
            "fullWidth": self.full_width,
            "min": self.min,
            "max": self.max,
            "currency": self.currency,
            "disabled": self.disabled,
            "errors": self.errors,
        }

    def serialize_value(self) -> Optional[float]:
        return self.value if self.value != None else self.empty_value



from typing import List
from ..widget_base import Input, NumberValueHandler


class NumberInput(Input):
    type = "number-input"
    empty_value = None

    def __init__(self, key: str, label: str, **kwargs):
        super().__init__(key)
        self.set_props(dict(label=label, **kwargs))

    def set_props(self, props):
        self.label = props.get("label", "Label")
        self.value = props.get("initial_value", 0)
        self.required = props.get("required", True)
        self.hint = props.get("hint", None)
        self.placeholder = props.get("placeholder", "")
        self.full_width = props.get("full_width", False)
        self.min = props.get("min")
        self.max = props.get("max")
        self.disabled = props.get("disabled", False)
        self.number_value_handler = NumberValueHandler(
            min=self.min, max=self.max, required=self.required
        )

    def render(self, ctx: dict):
        return {
            "type": self.type,
            "key": self.key,
            "label": self.label,
            "value": self.serialize_value(),
            "placeholder": self.placeholder,
            "required": self.required,
            "hint": self.hint,
            "fullWidth": self.full_width,
            "min": self.min,
            "max": self.max,
            "disabled": self.disabled,
            "errors": self.errors,
        }

    def validate(self) -> List[str]:
        return self.number_value_handler.validate(self.value)

    def serialize_value(self) -> float:
        return self.value



import os, re, pathlib
from setuptools import setup, find_packages

regex = r"^v(\d+\.\d+\.\d+)$"
TAG = os.getenv("TAG", "v0.0.0")
if not TAG or not re.search(regex, TAG):
    raise ValueError("TAG environment variable must be in the format v1.2.3")
match = re.search(regex, TAG)
version = match.group(1) if match else "0.0.0"

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text(encoding="utf-8")

# The list of requirements
requirements = (HERE / "requirements.txt").read_text(encoding="utf-8").split("\n")

setup(
    name="abstra",
    license="MIT",
    version=version,
    description="Abstra Lib",
    python_requires=">=3.8, <4",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/abstra-app/abstra-lib",
    entry_points={
        "console_scripts": [
            "abstra=abstra.cli:main",
        ],
    },
    package_data={
        "abstra_statics": ["dist/**/*", "dist/*"],
        "abstra_internals": ["templates/*"],
    },
    packages=find_packages(exclude=["tests"]),
    install_requires=requirements,
)



from ..widget_base import Input, AbstraOption, OptionsHandler
from typing import Union, List, Dict


class ChecklistInput(Input):
    type = "checklist-input"
    empty_value: List[str] = []

    def __init__(self, key: str, label: str, options: List[AbstraOption], **kwargs):
        super().__init__(key)
        self.set_props(dict(label=label, options=options, **kwargs))

    def set_props(self, props):
        self.label = props.get("label", "Label")
        self.options = props.get("options", [])
        self.value = props.get("initial_value", self.empty_value)
        self.required = props.get("required", True)
        self.hint = props.get("hint", None)
        self.full_width = props.get("full_width", False)
        self.disabled = props.get("disabled", False)
        self.min = props.get("min", 0)
        self.max = props.get("max", len(self.options))
        self.options_handler = OptionsHandler(self.options)

    def is_value_unset(self):
        return False

    def _validate_required(self) -> List[str]:
        if not self.required and len(self.value) == 0:
            return []
        errors = []
        if len(self.value) < self.min:
            if len(self.options) == 1:
                errors.append("i18n_error_required_field")
            else:
                errors.append("i18n_error_min_list")
        if len(self.value) > self.max:
            errors.append("i18n_error_max_list")
        return errors

    def render(self, ctx: dict):
        return {
            "type": self.type,
            "key": self.key,
            "options": self.options_handler.serialized_options(),
            "label": self.label,
            "value": self.serialize_value(),
            "required": self.required,
            "hint": self.hint,
            "fullWidth": self.full_width,
            "disabled": self.disabled,
            "errors": self.errors,
            "min": self.min,
            "max": self.max,
        }

    def serialize_value(self) -> List:
        return [self.options_handler.uuid_from_value(value) for value in self.value]

    def parse_value(self, value):
        return [self.options_handler.value_from_uuid(uuid) for uuid in value]
